<?php
namespace Ep\TecDoc\Middleware;

use Ep\Middleware\BaseUrl;
use Ep\TecDoc\Record\ManufacturersRecord;
use Ep\TecDoc\Record\ModelSeriesRecord;
use Ep\TecDoc\Record\VehicleByIdRecord;
use Ep\TecDoc\Record\VehicleByIds4Record;
use Ep\TecDoc\TecDoc;
use Ep\TecDoc\TecDocService;
use Ep\TecDoc\View\Helper\ConstructionYear;
use Ep\Wp\WordPress;
use Laminas\Diactoros\Response\EmptyResponse;
use Laminas\Diactoros\Response\JsonResponse;
use Laminas\Diactoros\Response\TextResponse;
use Laminas\Hydrator\ClassMethodsHydrator;
use Laminas\Hydrator\NamingStrategy\MapNamingStrategy;
use Laminas\Hydrator\Strategy\HydratorStrategy;
use Psr\Container\ContainerInterface;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;
use Laminas\Stratigility\MiddlewarePipe;
use Laminas\Stratigility\MiddlewarePipeInterface;
use function Laminas\Stratigility\path;

class TDataMiddleware implements MiddlewarePipeInterface
{
    /** @var MiddlewarePipeInterface */
    protected $pipeline;
    protected TecDoc $tecdoc;

    /**
     * TecDocMiddleware constructor.
     *
     * @param TecDoc $tecdoc
     */
    public function __construct(ContainerInterface $container)
    {
        $this->pipeline = new MiddlewarePipe();
        $this->tecdoc = $container->get('tecdoc_service');

        if (!is_dir('public/tdata')) {
            mkdir('public/tdata');
        }
    }

    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        $path = ltrim($request->getUri()->getPath(), '/');

        if ($path === 'manufacturers.json') {
            $data = $this->manufacturers();
        } else if (preg_match('#models/(\d+)\.json#', $path, $matches)) {
            $data = $this->models(intval($matches[1]));
        } else if (preg_match('#vehicles/(\d+)/(\d+)\.json#', $path, $matches)) {
            $data = $this->vehicles(intval($matches[1]), intval($matches[2]));
        } else if (preg_match('#vehicle/(\d+)\.json#', $path, $matches)) {
            $data = $this->vehicle(intval($matches[1]));
        } else {
            return new EmptyResponse(404);
        }

        return new JsonResponse($data);
    }

    private function manufacturers(): array
    {
        $featured = $this->tecdoc->getFeaturedManufacturers();
        $manufacturers = $this->tecdoc->getManufacturers();

        $extractRecords = fn (array $records) => array_values(
            array_map(
                fn (ManufacturersRecord $record) => [
                    'manuId' => $record->getId(),
                    'manuName' => $record->getName(),
                ],
                $records
            )
        );

        $data = [
            'featured' => $extractRecords($featured),
            'manufacturers' => $extractRecords($manufacturers),
        ];

        file_put_contents('public/tdata/manufacturers.json', json_encode($data));
        return $data;
    }

    private function models(int $manufacturerId): array
    {
        $models = $this->tecdoc->getModelSeries($manufacturerId);
        $yearHelper = new ConstructionYear();

        $data = array_values(
            array_map(
                fn (ModelSeriesRecord $record) => [
                    'modelId' => $record->getId(),
                    'modelname' => $record->getName(),
                    'yearOfConstrFrom' => $record->getYearOfConstrFrom(),
                    'yearOfConstrTo' => $record->getYearOfConstrTo(),
                    'year' => $yearHelper($record->getConstructionPeriod()),
                ],
                $models
            )
        );

        if (!is_dir('public/tdata/models')) {
            mkdir('public/tdata/models');
        }

        file_put_contents("public/tdata/models/$manufacturerId.json", json_encode($data));
        return $data;
    }

    private function vehicles(int $manufacturerId, int $modelId): array
    {
        $vehicles = $this->tecdoc->getVehicles($manufacturerId, $modelId, true);
        $hydrator = new ClassMethodsHydrator(false);
        $hydrator->setNamingStrategy(MapNamingStrategy::createFromExtractionMap([
            'id' => 'carId',
            'manufacturerId' => 'manuId',
            'manufacturerName' => 'manuName',
            'modelId' => 'modId',

        ]));
        $hydrator->addStrategy('vehicleDetails', new HydratorStrategy($hydrator, VehicleByIdRecord::class));

        $data = array_values(
            array_map(
                function (VehicleByIds4Record $record) use ($hydrator) {
                    return $hydrator->extract($record);
                },
                $vehicles
            )
        );

        if (!is_dir('public/tdata/vehicles')) {
            mkdir('public/tdata/vehicles');
        }

        if (!is_dir("public/tdata/vehicles/$manufacturerId")) {
            mkdir("public/tdata/vehicles/$manufacturerId");
        }

        file_put_contents("public/tdata/vehicles/$manufacturerId/$modelId.json", json_encode($data));
        return $data;
    }

    private function vehicle(int $vehicleId): array
    {
        $vehicles = $this->tecdoc->getVehicleByIds4([$vehicleId]);
        $hydrator = new ClassMethodsHydrator(false);
        $hydrator->setNamingStrategy(MapNamingStrategy::createFromExtractionMap([
            'id' => 'carId',
            'manufacturerId' => 'manuId',
            'manufacturerName' => 'manuName',
            'modelId' => 'modId',

        ]));
        $hydrator->addStrategy('vehicleDetails', new HydratorStrategy($hydrator, VehicleByIdRecord::class));

        $data = array_values(
            array_map(
                function (VehicleByIds4Record $record) use ($hydrator) {
                    return $hydrator->extract($record);
                },
                $vehicles
            )
        );

        if (!is_dir('public/tdata/vehicle')) {
            mkdir('public/tdata/vehicle');
        }

        $data = $data[0] ?? [];
        file_put_contents("public/tdata/vehicle/$vehicleId.json", json_encode($data));
        return $data;
    }

    public function pipe(MiddlewareInterface $middleware): void
    {
        $this->pipeline->pipe($middleware);
    }

    public function handle(ServerRequestInterface $request): ResponseInterface
    {
        if ($this->wp) {
            $this->wp->init($request->getAttribute(BaseUrl::BASE_URL));
        }

        return $this->pipeline->handle($request);
    }
}
